@@ -3,38 +3,50 @@
 """This module provides high-level classes to open and read DarwinCore Archive (DwC-A) files."""
 
 import os
+import zipfile
+import tarfile
 from tempfile import mkdtemp
-from zipfile import ZipFile
 from shutil import rmtree
 from errno import ENOENT
 
 import xml.etree.ElementTree as ET
 
 from dwca.utils import _DataFile
-from dwca.descriptors import ArchiveDescriptor
-from dwca.exceptions import RowNotFound, InvalidArchive
+from dwca.descriptors import ArchiveDescriptor, DataFileDescriptor
+from dwca.exceptions import RowNotFound, InvalidArchive, InvalidSimpleArchive
+
+DEFAULT_METADATA_FILENAME = "EML.xml"
+METAFILE_NAME = "meta.xml"
 
 
 class DwCAReader(object):
     """This class is used to represent a Darwin Core Archive as a whole.
 
-    It gives read access to the contained data, to the scientific metadata, ...
+    It gives read access to the contained data, to the scientific metadata, ... It supports
+    archives with or without Metafiles, such as described on page 2 of the Reference Guide
+    to the XML Descriptor (http://www.gbif.org/resource/80639).
 
-    :param path: path to the Darwin Core Archive (either a zip file or a directory) to open.
+    :param path: path to the Darwin Core Archive (either a zip/tgz file or a directory) to open.
     :type path: str
+    :param extensions_to_ignore: relative path (within the archive) of extension data files to\
+    ignore. This will improve performances and memory consumption with large archives.
+    :type extensions_to_ignore: list
 
-    A short usage example::
+    :raises: :class:`dwca.exceptions.InvalidArchive`
+    :raises: :class:`dwca.exceptions.InvalidSimpleArchive`
 
-        from dwca import DwCAReader
+    A simple usage example::
+
+        from dwca.read import DwCAReader
 
         dwca = DwCAReader('my_archive.zip')
         # Iterating on core rows is easy:
         for core_row in dwca:
-            # core_row is an instance of rows.CoreRow
-            print core_row
+            # core_row is an instance of dwca.rows.CoreRow
+            print(core_row)
 
         # Scientific metadata (EML) is available as an ElementTree.Element object
-        print dwca.metadata.prettify()
+        print(dwca.metadata)
 
         # Close the archive to free resources
         dwca.close()
@@ -44,7 +56,7 @@ class DwCAReader(object):
 
     ::
 
-        from dwca import DwCAReader
+        from dwca.read import DwCAReader
 
         with DwCAReader('my-archive.zip') as dwca:
             pass  # Do what you want
@@ -60,37 +72,56 @@ class DwCAReader(object):
         self.close()
 
     def __init__(self, path, extensions_to_ignore=None):
-        """Open the file, reads all metadata and store it in self.metadata.
-
-        Also already open the core file so we've a file descriptor for further access.
-        """
+        """Open the Darwin Core Archive."""
         if extensions_to_ignore is None:
             extensions_to_ignore = []
 
         #: The path to the Darwin Core Archive file, as passed to the constructor.
         self.archive_path = path
 
-        if os.path.isdir(self.archive_path):  # Archive as a directly readable directory
+        if os.path.isdir(self.archive_path):  # Archive is a (directly readable) directory
             self._workin_directory_path = self.archive_path
             self._directory_to_clean = None
-        else:  # Archive is zipped, we have to unzip it
-            self._directory_to_clean, self._workin_directory_path = self._unzip()
+        else:  # Archive is zipped/tgzipped, we have to extract it first.
+            self._directory_to_clean, self._workin_directory_path = self._extract()
 
         #: An :class:`descriptors.ArchiveDescriptor` instance giving access to the archive
         #: descriptor (``meta.xml``)
-        self.descriptor = ArchiveDescriptor(self._read_additional_file('meta.xml'),
-                                            files_to_ignore=extensions_to_ignore)
+        try:
+            self.descriptor = ArchiveDescriptor(self._read_additional_file(METAFILE_NAME),
+                                                files_to_ignore=extensions_to_ignore)
+        except IOError as e:
+            if e.errno == ENOENT:
+                self.descriptor = None
 
         #: A :class:`xml.etree.ElementTree.Element` instance containing the (scientific) metadata
-        #: of the archive.
+        #: of the archive, or None if the Archive contains no metadata.
         self.metadata = self._parse_metadata_file()
         #:
         self.source_metadata = None
 
-        self._corefile = _DataFile(self.descriptor.core,
-                                   self._workin_directory_path)
-        self._extensionfiles = [_DataFile(d, self._workin_directory_path)
-                                for d in self.descriptor.extensions]
+        if self.descriptor:
+            #  We have an Archive descriptor that we can use to access data files.
+            self._corefile = _DataFile(self._workin_directory_path, self.descriptor.core)
+            self._extensionfiles = [_DataFile(work_directory=self._workin_directory_path,
+                                              file_descriptor=d)
+                                    for d in self.descriptor.extensions]
+        else:  # Archive without descriptor, we'll have to find and inspect the data file
+            try:
+                datafile_name = self._is_valid_simple_archive()
+                d = DataFileDescriptor.make_from_file(os.path.join(self._workin_directory_path, datafile_name))
+
+                self._corefile = _DataFile(work_directory=self._workin_directory_path,
+                                           file_descriptor=d)
+                self._extensionfiles = []
+            except InvalidSimpleArchive:
+                msg = "No metafile was found, but archive includes multiple files/directories."
+                raise InvalidSimpleArchive(msg)
+
+    @property
+    def use_extensions(self):
+        """Return True if the Archive makes use of extensions."""
+        return (self.descriptor is not None) and self.descriptor.extensions
 
     @property
     # TODO: decide, test and document what we guarantee about ordering
@@ -150,62 +181,112 @@ class DwCAReader(object):
         .. note::
             - This method allows raw access to the files contained in the archive. It is for\
             example useful to open additional, non-standard files embedded in the archive.
-            - The file at this path is temporary and will be removed when closing the instance.
+            - If the archive is contained in a zip or tgz file, the returned path will point to a\
+            temporary file that will be removed when closing the :class:`dwca.read.DwCAReader`\
+            instance.
             - File existence is not tested.
 
         """
         return os.path.abspath(os.path.join(self._workin_directory_path, relative_path))
 
+    def _is_valid_simple_archive(self):
+        # If the working dir appear to contains a valid simple darwin core archive
+        # (one single data file + possibly some metadata), returns the name of the data file.
+        #
+        # Otherwise, throws an InvalidSimpleArchive exception.
+        path, dirs, files = next(os.walk(self._workin_directory_path))
+
+        if len(files) == 1:  # We found a single file
+            return files[0]
+        elif len(files) == 2:
+            # Two files found: if one of them is EML.xml, the other is considered as the data file
+            if DEFAULT_METADATA_FILENAME in files:
+                return [f for f in files if f is not DEFAULT_METADATA_FILENAME][0]
+            else:
+                invalid = True
+        else:
+            invalid = True
+
+        if invalid:
+            raise InvalidSimpleArchive()
+
     def _read_additional_file(self, relative_path):
         """Read an additional file in the archive and return its content."""
         p = self.absolute_temporary_path(relative_path)
         return open(p).read()
 
     def _parse_metadata_file(self):
-        """Load the archive (scientific) Metadata file, parse it with
-        ElementTree and return its content.
+        """Load the archive (scientific) Metadata file, parse it with\
+        ElementTree and return its content (or None if the Archive contains no metadata).
 
         :raises: :class:`dwca.exceptions.InvalidArchive` if the archive references an inexisting
         metadata file.
         """
-        fn = self.descriptor.metadata_filename
+        # If the archive has descriptor, look for the metadata filename there.
+        if self.descriptor and self.descriptor.metadata_filename:
+            fn = self.descriptor.metadata_filename
 
-        try:
-            return self._parse_xml_included_file(fn)
-        except IOError as e:
-            if e.errno == ENOENT:  # File not found
-                msg = "{} is referenced in the archive descriptor but missing.".format(fn)
-                raise InvalidArchive(msg)
+            try:
+                return self._parse_xml_included_file(fn)
+            except IOError as e:
+                if e.errno == ENOENT:  # File not found
+                    msg = "{} is referenced in the archive descriptor but missing.".format(fn)
+                    raise InvalidArchive(msg)
+
+        else:  # Otherwise, the metadata file has to be named 'EML.xml'
+            try:
+                return self._parse_xml_included_file(DEFAULT_METADATA_FILENAME)
+            except IOError as e:
+                if e.errno == ENOENT:  # File not found, this is an archive without metadata
+                    return None
 
     def _parse_xml_included_file(self, relative_path):
         """Load, parse and returns (as ElementTree.Element) XML file located at relative_path."""
         return ET.fromstring(self._read_additional_file(relative_path))
 
-    def _unzip(self):
-        """Unzip the current archive in a temporary directory and return paths.
+    def _unzip_or_untar(self):
+        """Create a temporary dir. and uncompress/unarchive self.archive_path there.
+
+        Returns the path to that temporary directory.
+
+        Raises InvalidArchive if not a zip nor a tgz file.
+        """
+        tmp_dir = mkdtemp()[1]  # Creating a temporary directory
+
+        # We first try to unzip (most common archives)
+        try:
+            # Security note: with Python < 2.7.4, a zip file may be able to write outside of the
+            # directory using absolute paths, parent (..) path, ... See note in ZipFile.extract doc
+            zipfile.ZipFile(self.archive_path, 'r').extractall(tmp_dir)
+        except zipfile.BadZipfile:
+            # Doesn't look like a valid zip, let's see if it's a tar archive (possibly compressed)
+            try:
+                tarfile.open(self.archive_path, 'r:*').extractall(tmp_dir)
+            except tarfile.ReadError:
+                raise InvalidArchive("The archive cannot be read. Is it a .zip or .tgz file?")
+
+        return tmp_dir
+
+    def _extract(self):
+        """Extract the current (Zip of Tar) archive in a temporary directory and return paths.
 
         Returns (path_to_clean_afterwards, path_to_content)
         """
-        unzipped_folder = mkdtemp()[1]  # Creating a temporary folder
-        # TODO: check content of file!!!! It may, for example contains
-        # absolute path (see zipfile doc)
-        ZipFile(self.archive_path, 'r').extractall(unzipped_folder)
-
+        extracted_dir = self._unzip_or_untar()
+        content = os.listdir(extracted_dir)
         # If the archive contains a single directory, we assume the real content is indeed under
         # this directory.
         #
         # See https://github.com/BelgianBiodiversityPlatform/python-dwca-reader/issues/49
-        content = os.listdir(unzipped_folder)
-
-        if len(content) == 1 and os.path.isdir(os.path.join(unzipped_folder, content[0])):
-            content_folder = os.path.join(unzipped_folder, content[0])
+        if len(content) == 1 and os.path.isdir(os.path.join(extracted_dir, content[0])):
+            content_dir = os.path.join(extracted_dir, content[0])
         else:
-            content_folder = unzipped_folder
+            content_dir = extracted_dir
 
-        return (unzipped_folder, content_folder)
+        return (extracted_dir, content_dir)
 
     def close(self):
-        """Close the Darwin Core Archive and cleanup temporary/working files.
+        """Close the Darwin Core Archive and remove temporary/working files.
 
         .. note::
             - Alternatively, :class:`.DwCAReader` can be instanciated using the `with` statement.\
@@ -213,14 +294,14 @@ class DwCAReader(object):
 
         """
         if self._directory_to_clean:
-            self._cleanup_temporary_folder()
+            self._cleanup_temporary_dir()
 
-    def _cleanup_temporary_folder(self):
+    def _cleanup_temporary_dir(self):
         rmtree(self._directory_to_clean, False)
 
     def core_contains_term(self, term_url):
         """Return True if the Core file of the archive contains the term_url term."""
-        return term_url in self.descriptor.core.terms
+        return term_url in self._corefile.file_descriptor.terms
 
     def __iter__(self):
         self._corefile_pointer = 0
@@ -243,11 +324,9 @@ class DwCAReader(object):
 
 
 class GBIFResultsReader(DwCAReader):
-
-    """This class is used to represent the slightly augmented variant of Darwin Core Archive
+    """This class is used to represent the slightly augmented variant of Darwin Core Archive\
     produced by the new GBIF Data Portal when exporting occurrences.
 
-
     It provides a few additions to :class:`.DwCAReader` that reflect the additional data provided
     in these specific archives:
 
@@ -263,14 +342,14 @@ class GBIFResultsReader(DwCAReader):
         #: {'dataset_uuid': 'dataset_metadata', ...}
         self.source_metadata = self._dataset_metadata_to_dict('dataset')
 
-    def _dataset_metadata_to_dict(self, folder):
-        dataset_dir = os.path.join(self._workin_directory_path, folder)
+    def _dataset_metadata_to_dict(self, directory):
+        dataset_dir = os.path.join(self._workin_directory_path, directory)
 
         r = {}
         for f in os.listdir(dataset_dir):
             if os.path.isfile(os.path.join(dataset_dir, f)):
                 key = os.path.splitext(f)[0]
-                r[key] = self._parse_xml_included_file(os.path.join(folder, f))
+                r[key] = self._parse_xml_included_file(os.path.join(directory, f))
         return r
 
     # Compared to a standard DwC-A, GBIF results export contains
