@@ -5,7 +5,6 @@ import unittest
 import os
 import tempfile
 
-from zipfile import BadZipfile
 import xml.etree.ElementTree as ET
 
 from dwca.read import DwCAReader, GBIFResultsReader
@@ -18,18 +17,51 @@ from .helpers import (GBIF_RESULTS_PATH, BASIC_ARCHIVE_PATH, EXTENSION_ARCHIVE_P
                       MULTIEXTENSIONS_ARCHIVE_PATH, NOHEADERS1_PATH, NOHEADERS2_PATH,
                       IDS_ARCHIVE_PATH, DEFAULT_VAL_PATH, UTF8EOL_ARCHIVE_PATH,
                       DIRECTORY_ARCHIVE_PATH, DEFAULT_META_VALUES, INVALID_LACKS_METADATA,
-                      SUBFOLDER_ARCHIVE_PATH)
+                      SUBDIR_ARCHIVE_PATH, SIMPLE_CSV, SIMPLE_CSV_EML, SIMPLE_CSV_DOS,
+                      BASIC_ENCLOSED_ARCHIVE_PATH, INVALID_SIMPLE_TOOMUCH, INVALID_SIMPLE_TWO,
+                      SIMPLE_CSV_NOTENCLOSED, NOMETADATA_PATH, DEFAULT_METADATA_FILENAME,
+                      BASIC_ARCHIVE_TGZ_PATH)
 
 
 class TestDwCAReader(unittest.TestCase):
     # TODO: Move row-oriented tests to another test class
     """Unit tests for DwCAReader class."""
 
-    def test_subfolder_archive(self):
+    def test_use_extensions(self):
+        """Ensure the .use_extensions attribute of DwCAReader works as intended."""
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            self.assertFalse(dwca.use_extensions)  # Basic archive without extensions
+
+        with DwCAReader(SIMPLE_CSV) as dwca:  # Just a CSV file, so no extensions
+            self.assertFalse(dwca.use_extensions)
+
+        with DwCAReader(EXTENSION_ARCHIVE_PATH) as dwca:
+            self.assertTrue(dwca.use_extensions)
+
+        with DwCAReader(MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
+            self.assertTrue(dwca.use_extensions)
+
+        with DwCAReader(EXTENSION_ARCHIVE_PATH, extensions_to_ignore="vernacularname.txt") as dwca:
+            # We ignore the extension, so archive appears without
+            self.assertFalse(dwca.use_extensions)
+
+    def test_default_metadata_filename(self):
+        """Ensure that metadata is found by it's default name.
+
+        Metadata is named "EML.xml", but no metadata attribute in Metafile.
+        """
+        with DwCAReader(DEFAULT_METADATA_FILENAME) as dwca:
+            self.assertIsInstance(dwca.metadata, ET.Element)
+
+            v = (dwca.metadata.find('dataset').find('creator').find('individualName')
+                     .find('givenName').text)
+            self.assertEqual(v, 'Nicolas')
+
+    def test_subdirectory_archive(self):
         """Ensure we support Archives where all the content is under a single directory."""
         num_files_before = len(os.listdir('.'))
         num_files_during = None
-        with DwCAReader(SUBFOLDER_ARCHIVE_PATH) as dwca:
+        with DwCAReader(SUBDIR_ARCHIVE_PATH) as dwca:
             # Ensure we have access to metadata
             self.assertIsInstance(dwca.metadata, ET.Element)
 
@@ -48,8 +80,8 @@ class TestDwCAReader(unittest.TestCase):
         self.assertEqual(num_files_before + 1, num_files_during)
         self.assertEqual(num_files_before, num_files_after)
 
-    def test_exception_invalid_archives(self):
-        """Ensure an InvalidArchive exception is raised when the archive is invalid."""
+    def test_exception_invalid_archives_missing_metadata(self):
+        """Ensure an exception is raised when referencing a missing metadata file."""
         # Sometimes, the archive metafile references a metadata file that's not present in the
         # archive. See for example http://dev.gbif.org/issues/browse/PF-2125
         with self.assertRaises(InvalidArchive) as cm:
@@ -61,7 +93,22 @@ class TestDwCAReader(unittest.TestCase):
         expected_message = "eml.xml is referenced in the archive descriptor but missing."
         self.assertEqual(str(the_exception), expected_message)
 
-        # TODO: Once implemented, test here other cases of InvalidArchive exceptions.
+    def test_exception_invalid_simple_archives(self):
+        """Ensure an exception is raised when simple archives can't be interpreted.
+
+        When there's no metafile in an archive, this one consists of a single data core file,
+        and possibly some metadata in EML.xml. If the archive doesn't follow this structure,
+        python-dwca-reader can't detect the data file and should throw an InvalidArchive exception.
+        """
+        # There's a random file (in addition to data and EML.xml) in this one, so we can't choose
+        # which file is the datafile.
+        with self.assertRaises(InvalidArchive):
+            a = DwCAReader(INVALID_SIMPLE_TOOMUCH)
+            a.close()
+
+        with self.assertRaises(InvalidArchive):
+            a = DwCAReader(INVALID_SIMPLE_TWO)
+            a.close()
 
     def test_default_values_metafile(self):
         """
@@ -75,6 +122,69 @@ class TestDwCAReader(unittest.TestCase):
                 self.assertIsInstance(row, CoreRow)
 
             # And verify the values themselves:
+            # Test also "fieldsenclosedBy"?
+
+    def test_simplecsv_archive(self):
+        """Ensure the reader works with archives consiting of a single CSV file.
+
+        As described in page #2 of http://www.gbif.org/resource/80639, those archives consists
+        of a single core data file where the first line provides the names of the Darwin Core terms
+        represented in the published data. That also seems to match quite well the definition of
+        Simple Darwin Core expressed as text: http://rs.tdwg.org/dwc/terms/simple/index.htm.
+        """
+        with DwCAReader(SIMPLE_CSV) as dwca:
+            # Ensure we get the correct number of rows
+            self.assertEqual(len(dwca.rows), 3)
+            # Ensure we can access arbitrary data
+            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            # Archive descriptor should be None
+            self.assertIsNone(dwca.descriptor)
+            # (scientific) metadata should be None
+            self.assertIsNone(dwca.metadata)
+
+        # Let's do the same tests again but with DOS line endings in the data file
+        with DwCAReader(SIMPLE_CSV_DOS) as dwca:
+            # Ensure we get the correct number of rows
+            self.assertEqual(len(dwca.rows), 3)
+            # Ensure we can access arbitrary data
+            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            # Archive descriptor should be None
+            self.assertIsNone(dwca.descriptor)
+            # (scientific) metadata should be None
+            self.assertIsNone(dwca.metadata)
+
+        # And with a file where fields are not double quotes-enclosed:
+        with DwCAReader(SIMPLE_CSV_NOTENCLOSED) as dwca:
+            # Ensure we get the correct number of rows
+            self.assertEqual(len(dwca.rows), 3)
+            # Ensure we can access arbitrary data
+            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            # Archive descriptor should be None
+            self.assertIsNone(dwca.descriptor)
+            # (scientific) metadata should be None
+            self.assertIsNone(dwca.metadata)
+
+    def test_simplecsv_archive_eml(self):
+        """Test Archive witthout metafile, but containing metadata.
+
+        Similar to test_simplecsv_archive, except the archive also contains a Metadata file named
+        EML.xml. This correspond to the second case on page #2 of
+        http://www.gbif.org/resource/80639. The metadata file having the "standard name", it should
+        properly handled.
+
+        """
+        with DwCAReader(SIMPLE_CSV_EML) as dwca:
+            # Ensure we get the correct number of rows
+            self.assertEqual(len(dwca.rows), 3)
+            # Ensure we can access arbitrary data
+            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            # Archive descriptor should be None
+            self.assertIsNone(dwca.descriptor)
+            # (scientific) metadata is found
+            self.assertIsInstance(dwca.metadata, ET.Element)
+            # TODO: also access a metadata element to ensure this really works?
+            v = (dwca.metadata.find('dataset').find('language').text)
+            self.assertEqual(v, 'en')
 
     def test_unzipped_archive(self):
         """Ensure it works with non-zipped (directory) archives."""
@@ -86,6 +196,19 @@ class TestDwCAReader(unittest.TestCase):
             for row in dwca:
                 self.assertIsInstance(row, CoreRow)
 
+    def test_tgz_archives(self):
+        """Ensure the reader (basic features) works with a .tgz Archive."""
+        with DwCAReader(BASIC_ARCHIVE_TGZ_PATH) as dwca:
+            self.assertIsInstance(dwca.metadata, ET.Element)
+
+            for row in dwca:
+                self.assertIsInstance(row, CoreRow)
+
+            rows = list(dwca)
+            self.assertEqual(len(rows), 2)
+            self.assertEqual('Borneo', rows[0].data[qn('locality')])
+            self.assertEqual('Mumbai', rows[1].data[qn('locality')])
+
     def test_classic_opening(self):
         """Ensure it also works w/o the 'with' statement."""
         dwca = DwCAReader(BASIC_ARCHIVE_PATH)
@@ -210,8 +333,8 @@ class TestDwCAReader(unittest.TestCase):
         self.assertIsInstance(r.metadata, ET.Element)
         r.close()
 
-    def test_temporary_folder_zipped(self):
-        """Test a temporary folder is created during execution.
+    def test_temporary_dir_zipped(self):
+        """Test a temporary directory is created during execution.
 
         (complementay to test_cleanup()
         """
@@ -221,7 +344,7 @@ class TestDwCAReader(unittest.TestCase):
 
         self.assertEqual(num_files_before, num_files_during - 1)
 
-    def test_no_temporary_folder_directory(self):
+    def test_no_temporary_dir_directory(self):
         """If archive is a directory, no need to create temporary files."""
         num_files_before = len(os.listdir('.'))
         with DwCAReader(DIRECTORY_ARCHIVE_PATH):
@@ -229,12 +352,22 @@ class TestDwCAReader(unittest.TestCase):
 
         self.assertEqual(num_files_before, num_files_during)
 
+    def test_archives_without_metadata(self):
+        """Ensure we can deal with an archive containing a metafile, but no metadata."""
+        with DwCAReader(NOMETADATA_PATH) as dwca:
+            self.assertIsNone(dwca.metadata)
+
+            # But the data is nevertheless accessible
+            rows = list(dwca)
+            self.assertEqual(len(rows), 2)
+            self.assertEqual('Borneo', rows[0].data[qn('locality')])
+            self.assertEqual('Mumbai', rows[1].data[qn('locality')])
+
     def test_metadata(self):
         """A few basic tests on the metadata attribute
 
         TODO: split
         """
-
         with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
             # Assert metadata is an instance of ElementTree.Element
             self.assertIsInstance(dwca.metadata, ET.Element)
@@ -246,12 +379,16 @@ class TestDwCAReader(unittest.TestCase):
 
     def test_core_contains_term(self):
         """Test the core_contains_term method."""
-
         # Example file contains locality but no country
         with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
             self.assertTrue(dwca.core_contains_term(qn('locality')))
             self.assertFalse(dwca.core_contains_term(qn('country')))
 
+        # Also test it with a simple (= no metafile) archive
+        with DwCAReader(SIMPLE_CSV) as dwca:
+            self.assertTrue(dwca.core_contains_term('datasetkey'))
+            self.assertFalse(dwca.core_contains_term('trucmachin'))
+
     def test_ignore_header_lines(self):
         with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
             # The sample file has two real rows + 1 header line
@@ -350,6 +487,19 @@ class TestDwCAReader(unittest.TestCase):
             self.assertEqual('Borneo', rows[0].data[qn('locality')])
             self.assertEqual('Mumbai', rows[1].data[qn('locality')])
 
+    def test_enclosed_data(self):
+        """Ensure data is properly trimmed when fieldsEnclosedBy is in use."""
+        with DwCAReader(BASIC_ENCLOSED_ARCHIVE_PATH) as dwca:
+            rows = list(dwca)
+
+            # Locality is enclosed in "'" chars, they should be trimmed...
+            self.assertEqual('Borneo', rows[0].data[qn('locality')])
+            self.assertEqual('Mumbai', rows[1].data[qn('locality')])
+
+            # But family isn't, so it shouldn't be altered
+            self.assertEqual('Tetraodontidae', rows[0].data[qn('family')])
+            self.assertEqual('Osphronemidae', rows[1].data[qn('family')])
+
     def test_read_core_value_default(self):
         """Retrieve a (default) value from core
 
@@ -501,11 +651,11 @@ class TestDwCAReader(unittest.TestCase):
         with DwCAReader(EXTENSION_ARCHIVE_PATH) as star_dwca:
             self.assertEqual(None, star_dwca.rows[0].source_metadata)
 
-    def test_not_zipfile(self):
-        """ Ensure BadZipfile is raised when passed archive is not a zip file."""
+    def test_unknown_archive_format(self):
+        """ Ensure InvalidArchive is raised when passed file is not a .zip not .tgz."""
         invalid_origin_file = tempfile.NamedTemporaryFile()
 
-        with self.assertRaises(BadZipfile):
+        with self.assertRaises(InvalidArchive):
             with DwCAReader(invalid_origin_file.name):
                 pass
 
