@@ -13,22 +13,22 @@ class _DataFile(object):
     # TODO: Test this class
     # Not done yet cause issues there will probably make DwCAReader tests fails anyway
     # In the future it could make sense to make it public
-    def __init__(self, file_descriptor, unzipped_folder_path):
-        # unzipped_folder_path: absolute path to the directory containing the unzipped archive.
-
+    def __init__(self, work_directory, file_descriptor):
+        # work_directory: absolute path to the target directory (archive, previously
+        # extracted if necessary).
         self.file_descriptor = file_descriptor
-        self._unzipped_folder_path = unzipped_folder_path
 
-        self._core_fhandler = io.open(os.path.join(self._unzipped_folder_path,
+        self._core_fhandler = io.open(os.path.join(work_directory,
                                                    self.file_descriptor.file_location),
                                       mode='r',
-                                      encoding=self.file_descriptor.encoding,
+                                      encoding=self.file_descriptor.file_encoding,
                                       newline=self.file_descriptor.lines_terminated_by,
                                       errors='replace')
 
         # On init, we parse the file once to build an index of newlines (including lines to ignore)
         # that will make random access faster later on...
-        self._line_offsets = get_all_line_offsets(self._core_fhandler, self.file_descriptor.encoding)
+        self._line_offsets = get_all_line_offsets(self._core_fhandler,
+                                                  self.file_descriptor.file_encoding)
 
         self.lines_to_ignore = self.file_descriptor.lines_to_ignore
 
